# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import os

from .command import Command, default_bin


class Maven(Command):
    def __init__(self, maven_bin=None):
        self.bin = default_bin(maven_bin, "mvn")


maven = Maven()


class MavenDefinition:
    """ MavenDefinition captures the maven invocation arguments.

    It allows creating build directories with the same definition, e.g.
    ```
    build_1 = maven_def.build("/tmp/build-1")
    build_2 = maven_def.build("/tmp/build-2")

    ...

    build1.install()
    build2.install()
    """

    def __init__(self, source, build_definitions=None,
                 benchmark_definitions=None, env=None):
        """ Initialize a MavenDefinition

        Parameters
        ----------
        source : str
                 Source directory where the top-level pom.xml is
                 located. This is usually the root of the project.
        build_definitions: list(str), optional
        benchmark_definitions: list(str), optional
        """
        self.source = os.path.abspath(source)
        self.build_definitions = build_definitions if build_definitions else []
        self.benchmark_definitions =\
            benchmark_definitions if benchmark_definitions else []
        self.env = env

    @property
    def build_arguments(self):
        """" Return the arguments to maven invocation for build. """
        arguments = self.build_definitions + [
            "-B", "-DskipTests", "-Drat.skip=true",
            "-Dorg.slf4j.simpleLogger.log.org.apache.maven.cli.transfer."
            "Slf4jMavenTransferListener=warn",
            "-T", "2C", "install"
        ]
        return arguments

    def build(self, build_dir, force=False, cmd_kwargs=None, **kwargs):
        """ Invoke maven into a build directory.

        Parameters
        ----------
        build_dir : str
                    Directory in which the Maven build will be instantiated.
        force : bool
                not used now
        """
        if os.path.exists(build_dir):
            # Extra safety to ensure we're deleting a build folder.
            if not MavenBuild.is_build_dir(build_dir):
                raise FileExistsError(
                    "{} is not a maven build".format(build_dir)
                )

        cmd_kwargs = cmd_kwargs if cmd_kwargs else {}
        assert MavenBuild.is_build_dir(build_dir)
        maven(*self.build_arguments, cwd=build_dir, env=self.env, **cmd_kwargs)
        return MavenBuild(build_dir, definition=self, **kwargs)

    @property
    def list_arguments(self):
        """" Return the arguments to maven invocation for list """
        arguments = [
            "-Dskip.perf.benchmarks=false", "-Dbenchmark.list=-lp", "install"
        ]
        return arguments

    @property
    def benchmark_arguments(self):
        """" Return the arguments to maven invocation for benchmark """
        arguments = self.benchmark_definitions + [
            "-Dskip.perf.benchmarks=false", "-Dbenchmark.fork=1",
            "-Dbenchmark.jvmargs=\"-Darrow.enable_null_check_for_get=false "
            "-Darrow.enable_unsafe_memory_access=true\"",
            "install"
        ]
        return arguments

    def __repr__(self):
        return "MavenDefinition[source={}]".format(self.source)


class MavenBuild(Maven):
    """ MavenBuild represents a build directory initialized by maven.

    The build instance can be used to build/test/install. It alleviates the
    user to know which generator is used.
    """

    def __init__(self, build_dir, definition=None):
        """ Initialize a MavenBuild.

        The caller must ensure that maven was invoked in the build directory.

        Parameters
        ----------
        definition : MavenDefinition
                     The definition to build from.
        build_dir : str
                    The build directory to setup into.
        """
        assert MavenBuild.is_build_dir(build_dir)
        super().__init__()
        self.build_dir = os.path.abspath(build_dir)
        self.definition = definition

    @property
    def binaries_dir(self):
        return self.build_dir

    def run(self, *argv, verbose=False, cwd=None, **kwargs):
        extra = []
        if verbose:
            extra.append("-X")
        if cwd is None:
            cwd = self.build_dir
        # Commands must be ran under the directory where pom.xml exists
        return super().run(*extra, *argv, **kwargs, cwd=cwd)

    def build(self, *argv, verbose=False, **kwargs):
        definition_args = self.definition.build_arguments
        cwd = self.binaries_dir
        return self.run(*argv, *definition_args, verbose=verbose, cwd=cwd,
                        env=self.definition.env, **kwargs)

    def list(self, *argv, verbose=False, **kwargs):
        definition_args = self.definition.list_arguments
        cwd = self.binaries_dir + "/performance"
        return self.run(*argv, *definition_args, verbose=verbose, cwd=cwd,
                        env=self.definition.env, **kwargs)

    def benchmark(self, *argv, verbose=False, **kwargs):
        definition_args = self.definition.benchmark_arguments
        cwd = self.binaries_dir + "/performance"
        return self.run(*argv, *definition_args, verbose=verbose, cwd=cwd,
                        env=self.definition.env, **kwargs)

    @staticmethod
    def is_build_dir(path):
        """ Indicate if a path is Maven top directory.

        This method only checks for the existence of paths and does not do any
        validation whatsoever.
        """
        pom_xml = os.path.join(path, "pom.xml")
        performance_dir = os.path.join(path, "performance")
        return os.path.exists(pom_xml) and os.path.isdir(performance_dir)

    @staticmethod
    def from_path(path):
        """ Instantiate a Maven from a path.

        This is used to recover from an existing physical directory (created
        with or without Maven).

        Note that this method is not idempotent as the original definition will
        be lost.
        """
        if not MavenBuild.is_build_dir(path):
            raise ValueError("Not a valid MavenBuild path: {}".format(path))

        return MavenBuild(path, definition=None)

    def __repr__(self):
        return ("MavenBuild["
                "build = {},"
                "definition = {}]".format(self.build_dir,
                                          self.definition))



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import os
import re
from shutil import rmtree, which

from .command import Command, default_bin


class CMake(Command):
    def __init__(self, cmake_bin=None):
        self.bin = default_bin(cmake_bin, "cmake")

    @staticmethod
    def default_generator():
        """ Infer default generator.

        Gives precedence to ninja if there exists an executable named `ninja`
        in the search path.
        """
        found_ninja = which("ninja")
        return "Ninja" if found_ninja else "Unix Makefiles"


cmake = CMake()


class CMakeDefinition:
    """ CMakeDefinition captures the cmake invocation arguments.

    It allows creating build directories with the same definition, e.g.
    ```
    build_1 = cmake_def.build("/tmp/build-1")
    build_2 = cmake_def.build("/tmp/build-2")

    ...

    build1.all()
    build2.all()
    """

    def __init__(self, source, build_type="release", generator=None,
                 definitions=None, env=None):
        """ Initialize a CMakeDefinition

        Parameters
        ----------
        source : str
                 Source directory where the top-level CMakeLists.txt is
                 located. This is usually the root of the project.
        generator : str, optional
        definitions: list(str), optional
        env : dict(str,str), optional
              Environment to use when invoking cmake. This can be required to
              work around cmake deficiencies, e.g. CC and CXX.
        """
        self.source = os.path.abspath(source)
        self.build_type = build_type
        self.generator = generator if generator else cmake.default_generator()
        self.definitions = definitions if definitions else []
        self.env = env

    @property
    def arguments(self):
        """" Return the arguments to cmake invocation. """
        arguments = [
            "-G{}".format(self.generator),
        ] + self.definitions + [
            self.source
        ]
        return arguments

    def build(self, build_dir, force=False, cmd_kwargs=None, **kwargs):
        """ Invoke cmake into a build directory.

        Parameters
        ----------
        build_dir : str
                    Directory in which the CMake build will be instantiated.
        force : bool
                If the build folder exists, delete it before. Otherwise if it's
                present, an error will be returned.
        """
        if os.path.exists(build_dir):
            # Extra safety to ensure we're deleting a build folder.
            if not CMakeBuild.is_build_dir(build_dir):
                raise FileExistsError(
                    "{} is not a cmake build".format(build_dir)
                )
            if not force:
                raise FileExistsError(
                    "{} exists use force=True".format(build_dir)
                )
            rmtree(build_dir)

        os.mkdir(build_dir)

        cmd_kwargs = cmd_kwargs if cmd_kwargs else {}
        cmake(*self.arguments, cwd=build_dir, env=self.env, **cmd_kwargs)
        return CMakeBuild(build_dir, self.build_type, definition=self,
                          **kwargs)

    def __repr__(self):
        return "CMakeDefinition[source={}]".format(self.source)


CMAKE_BUILD_TYPE_RE = re.compile("CMAKE_BUILD_TYPE:STRING=([a-zA-Z]+)")


class CMakeBuild(CMake):
    """ CMakeBuild represents a build directory initialized by cmake.

    The build instance can be used to build/test/install. It alleviates the
    user to know which generator is used.
    """

    def __init__(self, build_dir, build_type, definition=None):
        """ Initialize a CMakeBuild.

        The caller must ensure that cmake was invoked in the build directory.

        Parameters
        ----------
        definition : CMakeDefinition
                     The definition to build from.
        build_dir : str
                    The build directory to setup into.
        """
        assert CMakeBuild.is_build_dir(build_dir)
        super().__init__()
        self.build_dir = os.path.abspath(build_dir)
        self.build_type = build_type
        self.definition = definition

    @property
    def binaries_dir(self):
        return os.path.join(self.build_dir, self.build_type)

    def run(self, *argv, verbose=False, **kwargs):
        cmake_args = ["--build", self.build_dir, "--"]
        extra = []
        if verbose:
            extra.append("-v" if self.bin.endswith("ninja") else "VERBOSE=1")
        # Commands must be ran under the build directory
        return super().run(*cmake_args, *extra,
                           *argv, **kwargs, cwd=self.build_dir)

    def all(self):
        return self.run("all")

    def clean(self):
        return self.run("clean")

    def install(self):
        return self.run("install")

    def test(self):
        return self.run("test")

    @staticmethod
    def is_build_dir(path):
        """ Indicate if a path is CMake build directory.

        This method only checks for the existence of paths and does not do any
        validation whatsoever.
        """
        cmake_cache = os.path.join(path, "CMakeCache.txt")
        cmake_files = os.path.join(path, "CMakeFiles")
        return os.path.exists(cmake_cache) and os.path.exists(cmake_files)

    @staticmethod
    def from_path(path):
        """ Instantiate a CMakeBuild from a path.

        This is used to recover from an existing physical directory (created
        with or without CMakeBuild).

        Note that this method is not idempotent as the original definition will
        be lost. Only build_type is recovered.
        """
        if not CMakeBuild.is_build_dir(path):
            raise ValueError("Not a valid CMakeBuild path: {}".format(path))

        build_type = None
        # Infer build_type by looking at CMakeCache.txt and looking for a magic
        # definition
        cmake_cache_path = os.path.join(path, "CMakeCache.txt")
        with open(cmake_cache_path, "r") as cmake_cache:
            candidates = CMAKE_BUILD_TYPE_RE.findall(cmake_cache.read())
            build_type = candidates[0].lower() if candidates else "release"

        return CMakeBuild(path, build_type)

    def __repr__(self):
        return ("CMakeBuild["
                "build = {},"
                "build_type = {},"
                "definition = {}]".format(self.build_dir,
                                          self.build_type,
                                          self.definition))



#!/usr/bin/env python3
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# Rename a bunch of corpus files to their SHA1 hashes, and
# pack them into a ZIP archive.

import hashlib
from pathlib import Path
import sys
import zipfile


def process_dir(corpus_dir, zip_output):
    seen = set()

    for child in corpus_dir.iterdir():
        if not child.is_file():
            raise IOError("Not a file: {0}".format(child))
        with child.open('rb') as f:
            data = f.read()
        arcname = hashlib.sha1(data).hexdigest()
        if arcname in seen:
            raise ValueError("Duplicate hash: {0} (in file {1})"
                             .format(arcname, child))
        zip_output.writestr(str(arcname), data)
        seen.add(arcname)


def main(corpus_dir, zip_output_name):
    with zipfile.ZipFile(zip_output_name, 'w') as zip_output:
        process_dir(Path(corpus_dir), zip_output)


if __name__ == "__main__":
    if len(sys.argv) != 3:
        print("Usage: {0} <corpus dir> <output zip file>".format(sys.argv[0]))
        sys.exit(1)
    main(sys.argv[1], sys.argv[2])



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import glob
import json
import os
import re

from .core import BenchmarkSuite
from .google import GoogleBenchmarkCommand, GoogleBenchmark
from .jmh import JavaMicrobenchmarkHarnessCommand, JavaMicrobenchmarkHarness
from ..lang.cpp import CppCMakeDefinition, CppConfiguration
from ..lang.java import JavaMavenDefinition, JavaConfiguration
from ..utils.cmake import CMakeBuild
from ..utils.maven import MavenBuild
from ..utils.logger import logger


def regex_filter(re_expr):
    if re_expr is None:
        return lambda s: True
    re_comp = re.compile(re_expr)
    return lambda s: re_comp.search(s)


DEFAULT_REPETITIONS = 1


class BenchmarkRunner:
    def __init__(self, suite_filter=None, benchmark_filter=None,
                 repetitions=DEFAULT_REPETITIONS):
        self.suite_filter = suite_filter
        self.benchmark_filter = benchmark_filter
        self.repetitions = repetitions

    @property
    def suites(self):
        raise NotImplementedError("BenchmarkRunner must implement suites")

    @staticmethod
    def from_rev_or_path(src, root, rev_or_path, cmake_conf, **kwargs):
        raise NotImplementedError(
            "BenchmarkRunner must implement from_rev_or_path")


class StaticBenchmarkRunner(BenchmarkRunner):
    """ Run suites from a (static) set of suites. """

    def __init__(self, suites, **kwargs):
        self._suites = suites
        super().__init__(**kwargs)

    @property
    def list_benchmarks(self):
        for suite in self._suites:
            for benchmark in suite.benchmarks:
                yield "{}.{}".format(suite.name, benchmark.name)

    @property
    def suites(self):
        suite_fn = regex_filter(self.suite_filter)
        benchmark_fn = regex_filter(self.benchmark_filter)

        for suite in (s for s in self._suites if suite_fn(s.name)):
            benchmarks = [b for b in suite.benchmarks if benchmark_fn(b.name)]
            yield BenchmarkSuite(suite.name, benchmarks)

    @classmethod
    def is_json_result(cls, path_or_str):
        builder = None
        try:
            builder = cls.from_json(path_or_str)
        except BaseException:
            pass

        return builder is not None

    @staticmethod
    def from_json(path_or_str, **kwargs):
        # .codec imported here to break recursive imports
        from .codec import BenchmarkRunnerCodec
        if os.path.isfile(path_or_str):
            with open(path_or_str) as f:
                loaded = json.load(f)
        else:
            loaded = json.loads(path_or_str)
        return BenchmarkRunnerCodec.decode(loaded, **kwargs)

    def __repr__(self):
        return "BenchmarkRunner[suites={}]".format(list(self.suites))


class CppBenchmarkRunner(BenchmarkRunner):
    """ Run suites from a CMakeBuild. """

    def __init__(self, build, **kwargs):
        """ Initialize a CppBenchmarkRunner. """
        self.build = build
        super().__init__(**kwargs)

    @staticmethod
    def default_configuration(**kwargs):
        """ Returns the default benchmark configuration. """
        return CppConfiguration(
            build_type="release", with_tests=False, with_benchmarks=True,
            with_compute=True,
            with_csv=True,
            with_dataset=True,
            with_json=True,
            with_parquet=True,
            with_python=False,
            with_brotli=True,
            with_bz2=True,
            with_lz4=True,
            with_snappy=True,
            with_zlib=True,
            with_zstd=True,
            **kwargs)

    @property
    def suites_binaries(self):
        """ Returns a list of benchmark binaries for this build. """
        # Ensure build is up-to-date to run benchmarks
        self.build()
        # Not the best method, but works for now
        glob_expr = os.path.join(self.build.binaries_dir, "*-benchmark")
        return {os.path.basename(b): b for b in glob.glob(glob_expr)}

    def suite(self, name, suite_bin):
        """ Returns the resulting benchmarks for a given suite. """
        suite_cmd = GoogleBenchmarkCommand(suite_bin, self.benchmark_filter)

        # Ensure there will be data
        benchmark_names = suite_cmd.list_benchmarks()
        if not benchmark_names:
            return None

        results = suite_cmd.results(repetitions=self.repetitions)
        benchmarks = GoogleBenchmark.from_json(results.get("benchmarks"))
        return BenchmarkSuite(name, benchmarks)

    @property
    def list_benchmarks(self):
        for suite_name, suite_bin in self.suites_binaries.items():
            suite_cmd = GoogleBenchmarkCommand(suite_bin)
            for benchmark_name in suite_cmd.list_benchmarks():
                yield "{}.{}".format(suite_name, benchmark_name)

    @property
    def suites(self):
        """ Returns all suite for a runner. """
        suite_matcher = regex_filter(self.suite_filter)

        suite_and_binaries = self.suites_binaries
        for suite_name in suite_and_binaries:
            if not suite_matcher(suite_name):
                logger.debug("Ignoring suite {}".format(suite_name))
                continue

            suite_bin = suite_and_binaries[suite_name]
            suite = self.suite(suite_name, suite_bin)

            # Filter may exclude all benchmarks
            if not suite:
                logger.debug("Suite {} executed but no results"
                             .format(suite_name))
                continue

            yield suite

    @staticmethod
    def from_rev_or_path(src, root, rev_or_path, cmake_conf, **kwargs):
        """ Returns a BenchmarkRunner from a path or a git revision.

        First, it checks if `rev_or_path` is a valid path (or string) of a json
        object that can deserialize to a BenchmarkRunner. If so, it initialize
        a StaticBenchmarkRunner from it. This allows memoizing the result of a
        run in a file or a string.

        Second, it checks if `rev_or_path` points to a valid CMake build
        directory.  If so, it creates a CppBenchmarkRunner with this existing
        CMakeBuild.

        Otherwise, it assumes `rev_or_path` is a revision and clone/checkout
        the given revision and create a fresh CMakeBuild.
        """
        build = None
        if StaticBenchmarkRunner.is_json_result(rev_or_path):
            return StaticBenchmarkRunner.from_json(rev_or_path, **kwargs)
        elif CMakeBuild.is_build_dir(rev_or_path):
            build = CMakeBuild.from_path(rev_or_path)
            return CppBenchmarkRunner(build, **kwargs)
        else:
            # Revisions can references remote via the `/` character, ensure
            # that the revision is path friendly
            path_rev = rev_or_path.replace("/", "_")
            root_rev = os.path.join(root, path_rev)
            os.mkdir(root_rev)

            clone_dir = os.path.join(root_rev, "arrow")
            # Possibly checkout the sources at given revision, no need to
            # perform cleanup on cloned repository as root_rev is reclaimed.
            src_rev, _ = src.at_revision(rev_or_path, clone_dir)
            cmake_def = CppCMakeDefinition(src_rev.cpp, cmake_conf)
            build_dir = os.path.join(root_rev, "build")
            return CppBenchmarkRunner(cmake_def.build(build_dir), **kwargs)


class JavaBenchmarkRunner(BenchmarkRunner):
    """ Run suites for Java. """

    # default repetitions is 5 for Java microbenchmark harness
    def __init__(self, build, **kwargs):
        """ Initialize a JavaBenchmarkRunner. """
        self.build = build
        super().__init__(**kwargs)

    @staticmethod
    def default_configuration(**kwargs):
        """ Returns the default benchmark configuration. """
        return JavaConfiguration(**kwargs)

    def suite(self, name):
        """ Returns the resulting benchmarks for a given suite. """
        # update .m2 directory, which installs target jars
        self.build.build()

        suite_cmd = JavaMicrobenchmarkHarnessCommand(
            self.build, self.benchmark_filter)

        # Ensure there will be data
        benchmark_names = suite_cmd.list_benchmarks()
        if not benchmark_names:
            return None

        results = suite_cmd.results(repetitions=self.repetitions)
        benchmarks = JavaMicrobenchmarkHarness.from_json(results)
        return BenchmarkSuite(name, benchmarks)

    @property
    def list_benchmarks(self):
        """ Returns all suite names """
        # Ensure build is up-to-date to run benchmarks
        self.build.build()

        suite_cmd = JavaMicrobenchmarkHarnessCommand(self.build)
        benchmark_names = suite_cmd.list_benchmarks()
        for benchmark_name in benchmark_names:
            yield "{}".format(benchmark_name)

    @property
    def suites(self):
        """ Returns all suite for a runner. """
        suite_name = "JavaBenchmark"
        suite = self.suite(suite_name)

        # Filter may exclude all benchmarks
        if not suite:
            logger.debug("Suite {} executed but no results"
                         .format(suite_name))
            return

        yield suite

    @staticmethod
    def from_rev_or_path(src, root, rev_or_path, maven_conf, **kwargs):
        """ Returns a BenchmarkRunner from a path or a git revision.

        First, it checks if `rev_or_path` is a valid path (or string) of a json
        object that can deserialize to a BenchmarkRunner. If so, it initialize
        a StaticBenchmarkRunner from it. This allows memoizing the result of a
        run in a file or a string.

        Second, it checks if `rev_or_path` points to a valid Maven build
        directory.  If so, it creates a JavaBenchmarkRunner with this existing
        MavenBuild.

        Otherwise, it assumes `rev_or_path` is a revision and clone/checkout
        the given revision and create a fresh MavenBuild.
        """
        if StaticBenchmarkRunner.is_json_result(rev_or_path):
            return StaticBenchmarkRunner.from_json(rev_or_path, **kwargs)
        elif MavenBuild.is_build_dir(rev_or_path):
            maven_def = JavaMavenDefinition(rev_or_path, maven_conf)
            return JavaBenchmarkRunner(maven_def.build(rev_or_path), **kwargs)
        else:
            # Revisions can references remote via the `/` character, ensure
            # that the revision is path friendly
            path_rev = rev_or_path.replace("/", "_")
            root_rev = os.path.join(root, path_rev)
            os.mkdir(root_rev)

            clone_dir = os.path.join(root_rev, "arrow")
            # Possibly checkout the sources at given revision, no need to
            # perform cleanup on cloned repository as root_rev is reclaimed.
            src_rev, _ = src.at_revision(rev_or_path, clone_dir)
            maven_def = JavaMavenDefinition(src_rev.java, maven_conf)
            build_dir = os.path.join(root_rev, "arrow/java")
            return JavaBenchmarkRunner(maven_def.build(build_dir), **kwargs)



#!/usr/bin/env python
# Copyright 2015 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os


# For readability purposes we define the error codes and messages at the top of the
# file. New codes and messages must be added here. Old error messages MUST NEVER BE
# DELETED, but can be renamed. The tuple layout for a new entry is: error code enum name,
# numeric error code, format string of the message.
#
# TODO Add support for SQL Error Codes
#      https://msdn.microsoft.com/en-us/library/ms714687%28v=vs.85%29.aspx
error_codes = (
  ("OK", 0, ""),

  ("UNUSED", 1, "<UNUSED>"),

  ("GENERAL", 2, "$0"),

  ("CANCELLED", 3, "$0"),

  ("ANALYSIS_ERROR", 4, "$0"),

  ("NOT_IMPLEMENTED_ERROR", 5, "$0"),

  ("RUNTIME_ERROR", 6, "$0"),

  ("MEM_LIMIT_EXCEEDED", 7, "$0"),

  ("INTERNAL_ERROR", 8, "$0"),

  ("RECOVERABLE_ERROR", 9, "$0"),

  ("PARQUET_MULTIPLE_BLOCKS", 10,
   "Parquet files should not be split into multiple hdfs-blocks. file=$0"),

  ("PARQUET_COLUMN_METADATA_INVALID", 11,
   "Column metadata states there are $0 values, but read $1 values from column $2. "
   "file=$3"),

  ("PARQUET_HEADER_PAGE_SIZE_EXCEEDED", 12, "(unused)"),

  ("PARQUET_HEADER_EOF", 13,
    "ParquetScanner: reached EOF while deserializing data page header. file=$0"),

  ("PARQUET_GROUP_ROW_COUNT_ERROR", 14,
    "Metadata states that in group $0($1) there are $2 rows, but $3 rows were read."),

  ("PARQUET_GROUP_ROW_COUNT_OVERFLOW", 15, "(unused)"),

  ("PARQUET_MISSING_PRECISION", 16,
   "File '$0' column '$1' does not have the decimal precision set."),

  ("PARQUET_WRONG_PRECISION", 17,
    "File '$0' column '$1' has a precision that does not match the table metadata "
    " precision. File metadata precision: $2, table metadata precision: $3."),

  ("PARQUET_BAD_CONVERTED_TYPE", 18,
   "File '$0' column '$1' does not have converted type set to DECIMAL"),

  ("PARQUET_INCOMPATIBLE_DECIMAL", 19,
   "File '$0' column '$1' contains decimal data but the table metadata has type $2"),

  ("SEQUENCE_SCANNER_PARSE_ERROR", 20,
   "Problem parsing file $0 at $1$2"),

  ("SNAPPY_DECOMPRESS_INVALID_BLOCK_SIZE", 21,
   "Decompressor: block size is too big.  Data is likely corrupt. Size: $0"),

  ("SNAPPY_DECOMPRESS_INVALID_COMPRESSED_LENGTH", 22,
   "Decompressor: invalid compressed length.  Data is likely corrupt."),

  ("SNAPPY_DECOMPRESS_UNCOMPRESSED_LENGTH_FAILED", 23,
   "Snappy: GetUncompressedLength failed"),

  ("SNAPPY_DECOMPRESS_RAW_UNCOMPRESS_FAILED", 24,
   "SnappyBlock: RawUncompress failed"),

  ("SNAPPY_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT", 25,
   "Snappy: Decompressed size is not correct."),

  ("HDFS_SCAN_NODE_UNKNOWN_DISK", 26, "Unknown disk id.  "
   "This will negatively affect performance. "
   "Check your hdfs settings to enable block location metadata."),

  ("FRAGMENT_EXECUTOR", 27, "Reserved resource size ($0) is larger than "
    "query mem limit ($1), and will be restricted to $1. Configure the reservation "
    "size by setting RM_INITIAL_MEM."),

  ("PARTITIONED_HASH_JOIN_MAX_PARTITION_DEPTH", 28,
   "Cannot perform join at hash join node with id $0."
   " The input data was partitioned the maximum number of $1 times."
   " This could mean there is significant skew in the data or the memory limit is"
   " set too low."),

  ("PARTITIONED_AGG_MAX_PARTITION_DEPTH", 29,
   "Cannot perform aggregation at hash aggregation node with id $0."
   " The input data was partitioned the maximum number of $1 times."
   " This could mean there is significant skew in the data or the memory limit is"
   " set too low."),

  ("MISSING_BUILTIN", 30, "Builtin '$0' with symbol '$1' does not exist. "
   "Verify that all your impalads are the same version."),

  ("RPC_GENERAL_ERROR", 31, "RPC Error: $0"),
  ("RPC_TIMEOUT", 32, "RPC timed out"),

  ("UDF_VERIFY_FAILED", 33,
   "Failed to verify function $0 from LLVM module $1, see log for more details."),

  ("PARQUET_CORRUPT_VALUE", 34, "File $0 corrupt. RLE level data bytes = $1"),

  ("AVRO_DECIMAL_RESOLUTION_ERROR", 35, "Column '$0' has conflicting Avro decimal types. "
   "Table schema $1: $2, file schema $1: $3"),

  ("AVRO_DECIMAL_METADATA_MISMATCH", 36, "Column '$0' has conflicting Avro decimal types. "
   "Declared $1: $2, $1 in table's Avro schema: $3"),

  ("AVRO_SCHEMA_RESOLUTION_ERROR", 37, "Unresolvable types for column '$0': "
   "table type: $1, file type: $2"),

  ("AVRO_SCHEMA_METADATA_MISMATCH", 38, "Unresolvable types for column '$0': "
   "declared column type: $1, table's Avro schema type: $2"),

  ("AVRO_UNSUPPORTED_DEFAULT_VALUE", 39, "Field $0 is missing from file and default "
   "values of type $1 are not yet supported."),

  ("AVRO_MISSING_FIELD", 40, "Inconsistent table metadata. Mismatch between column "
   "definition and Avro schema: cannot read field $0 because there are only $1 fields."),

  ("AVRO_MISSING_DEFAULT", 41,
   "Field $0 is missing from file and does not have a default value."),

  ("AVRO_NULLABILITY_MISMATCH", 42,
   "Field $0 is nullable in the file schema but not the table schema."),

  ("AVRO_NOT_A_RECORD", 43,
   "Inconsistent table metadata. Field $0 is not a record in the Avro schema."),

  ("PARQUET_DEF_LEVEL_ERROR", 44, "Could not read definition level, even though metadata"
   " states there are $0 values remaining in data page. file=$1"),

  ("PARQUET_NUM_COL_VALS_ERROR", 45, "Mismatched number of values in column index $0 "
   "($1 vs. $2). file=$3"),

  ("PARQUET_DICT_DECODE_FAILURE", 46, "Failed to decode dictionary-encoded value. "
   "file=$0"),

  ("SSL_PASSWORD_CMD_FAILED", 47,
   "SSL private-key password command ('$0') failed with error: $1"),

  ("SSL_CERTIFICATE_PATH_BLANK", 48, "The SSL certificate path is blank"),
  ("SSL_PRIVATE_KEY_PATH_BLANK", 49, "The SSL private key path is blank"),

  ("SSL_CERTIFICATE_NOT_FOUND", 50, "The SSL certificate file does not exist at path $0"),
  ("SSL_PRIVATE_KEY_NOT_FOUND", 51, "The SSL private key file does not exist at path $0"),

  ("SSL_SOCKET_CREATION_FAILED", 52, "SSL socket creation failed: $0"),

  ("MEM_ALLOC_FAILED", 53, "Memory allocation of $0 bytes failed"),

  ("PARQUET_REP_LEVEL_ERROR", 54, "Could not read repetition level, even though metadata"
   " states there are $0 values remaining in data page. file=$1"),

  ("PARQUET_UNRECOGNIZED_SCHEMA", 55, "File '$0' has an incompatible Parquet schema for "
   "column '$1'. Column type: $2, Parquet schema:\\n$3"),

  ("COLLECTION_ALLOC_FAILED", 56, "Failed to allocate buffer for collection '$0'."),

  ("TMP_DEVICE_BLACKLISTED", 57,
    "Temporary device for directory $0 is blacklisted from a previous error and cannot "
    "be used."),

  ("TMP_FILE_BLACKLISTED", 58,
    "Temporary file $0 is blacklisted from a previous error and cannot be expanded."),

  ("RPC_CLIENT_CONNECT_FAILURE", 59,
    "RPC client failed to connect: $0"),

  ("STALE_METADATA_FILE_TOO_SHORT", 60, "Metadata for file '$0' appears stale. "
   "Try running \\\"refresh $1\\\" to reload the file metadata."),

  ("PARQUET_BAD_VERSION_NUMBER", 61, "File '$0' has an invalid version number: $1\\n"
   "This could be due to stale metadata. Try running \\\"refresh $2\\\"."),

  ("SCANNER_INCOMPLETE_READ", 62, "Tried to read $0 bytes but could only read $1 bytes. "
   "This may indicate data file corruption. (file $2, byte offset: $3)"),

  ("SCANNER_INVALID_READ", 63, "Invalid read of $0 bytes. This may indicate data file "
   "corruption. (file $1, byte offset: $2)"),

  ("AVRO_BAD_VERSION_HEADER", 64, "File '$0' has an invalid version header: $1\\n"
   "Make sure the file is an Avro data file."),

  ("UDF_MEM_LIMIT_EXCEEDED", 65, "$0's allocations exceeded memory limits."),

  ("BTS_BLOCK_OVERFLOW", 66, "Cannot process row that is bigger than the IO size "
   "(row_size=$0, null_indicators_size=$1). To run this query, increase the IO size "
   "(--read_size option)."),

  ("COMPRESSED_FILE_MULTIPLE_BLOCKS", 67,
   "For better performance, snappy-, gzip-, and bzip-compressed files "
   "should not be split into multiple HDFS blocks. file=$0 offset $1"),

  ("COMPRESSED_FILE_BLOCK_CORRUPTED", 68,
   "$0 Data error, likely data corrupted in this block."),

  ("COMPRESSED_FILE_DECOMPRESSOR_ERROR", 69, "$0 Decompressor error at $1, code=$2"),

  ("COMPRESSED_FILE_DECOMPRESSOR_NO_PROGRESS", 70,
   "Decompression failed to make progress, but end of input is not reached. "
   "File appears corrupted. file=$0"),

  ("COMPRESSED_FILE_TRUNCATED", 71,
   "Unexpected end of compressed file. File may be truncated. file=$0")
)

# Verifies the uniqueness of the error constants and numeric error codes.
# Numeric codes must start from 0, be in order and have no gaps
def check_duplicates(codes):
  constants = {}
  next_num_code = 0
  for row in codes:
    if row[0] in constants:
      print("Constant %s already used, please check definition of '%s'!" % \
            (row[0], constants[row[0]]))
      exit(1)
    if row[1] != next_num_code:
      print("Numeric error codes must start from 0, be in order, and not have any gaps: "
            "got %d, expected %d" % (row[1], next_num_code))
      exit(1)
    next_num_code += 1
    constants[row[0]] = row[2]

preamble = """
// Copyright 2015 Cloudera Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//
// THIS FILE IS AUTO GENERATED BY generated_error_codes.py DO NOT MODIFY
// IT BY HAND.
//

namespace cpp impala
namespace java com.cloudera.impala.thrift

"""
# The script will always generate the file, CMake will take care of running it only if
# necessary.
target_file = os.path.join(sys.argv[1], "ErrorCodes.thrift")

# Check uniqueness of error constants and numeric codes
check_duplicates(error_codes)

fid = open(target_file, "w+")
try:
  fid.write(preamble)
  fid.write("""\nenum TErrorCode {\n""")
  fid.write(",\n".join(map(lambda x: "  %s = %d" % (x[0], x[1]), error_codes)))
  fid.write("\n}")
  fid.write("\n")
  fid.write("const list<string> TErrorMessage = [\n")
  fid.write(",\n".join(map(lambda x: "  // %s\n  \"%s\"" %(x[0], x[2]), error_codes)))
  fid.write("\n]")
finally:
  fid.close()

print("%s created." % target_file)
